@@ -5,6 +5,7 @@ Basically a wrapper around Coopr's `pyomo.ConcreteModel` class.
 import logging, time, weakref
 import coopr.pyomo as pyomo
 from coopr.opt.base import solvers as cooprsolver
+import pandas as pd
 
 # make pyomo recognize that True == 1
 pyomo.base.numvalue.KnownConstants[True]=pyomo.base.numvalue.NumericConstant(None, None, 1.0)
@@ -15,7 +16,10 @@ variable_kinds = dict(
     Boolean=pyomo.Boolean)
 
 from config import user_config
-from commonscripts import update_attributes
+from commonscripts import update_attributes, joindir
+
+def full_filename(filename):
+    return joindir(user_config.directory, filename)
 
 class OptimizationObject(object):
     '''
@@ -35,6 +39,14 @@ class OptimizationObject(object):
         self.init_optimization()
         
 
+#    def get(self, attr):
+#        '''
+#        convinience method for getting attributes an object may not have
+#        without throwing errors
+#        '''
+#        return getattr(self, attr, None)
+
+
     def init_optimization(self):
         '''
         Initialize optimization components: add a container for children.
@@ -131,11 +143,15 @@ class OptimizationObject(object):
 
 
     def add_parameter(self, name, index=None, values=None, mutable=True, default=None, **kwargs):
-        name=self._id(name)
-        self._parent_problem().add_component_to_problem(pyomo.Param(index,name=name,default=default, mutable=mutable, **kwargs))
+        name = self._id(name)
+        self._parent_problem().add_component_to_problem(
+            pyomo.Param(index, name=name, 
+                default=default, mutable=mutable, **kwargs))
         if values is not None:
-            var=self._parent_problem().get_component(name)
-            for i in index: var[i]=values[i]
+            if pd.Series(values).count() != len(values):
+                raise ValueError('a parameter value cannot be NaN')
+            var = self._parent_problem().get_component(name)
+            for i in index: var[i] = values[i]
         
     def add_constraint(self,name,time,expression): 
         '''Create a new constraint and add it to the object's constraints and the model's constraints.'''
@@ -376,19 +392,22 @@ class OptimizationProblem(OptimizationObject):
             logging.info('Problem solved in {}s.'.format(self.solution_time))
         
         if user_config.problem_filename:
-            logging.getLogger().setLevel(logging.CRITICAL) #disable coopr's funny loggings when writing lp files.  
-            self.write_model(user_config.problem_filename)
+            # disable coopr's funny loggings when writing lp files.  
+            logging.getLogger().setLevel(logging.CRITICAL) 
+            self.write_model(full_filename(user_config.problem_filename))
             logging.getLogger().setLevel(user_config.logging_level)
                 
         if not self.solved:
-            if self.stochastic_formulation: 
-                self._stochastic_instance.pprint('unsolved-stochastic-instance.txt')
-            self.write_model('unsolved-problem-formulation.lp')
+            if user_config.problem_filename:
+                if self.stochastic_formulation: 
+                    self._stochastic_instance.pprint(full_filename(
+                        'unsolved-stochastic-instance.txt'))
+                self.write_model(full_filename(
+                    'unsolved-problem-formulation.lp'))
             raise OptimizationError('problem not solved')
         
         instance.load(results, 
             allow_consistent_values_for_fixed_vars=True)
-#        instance._load_solution(results.solution(0), ignore_invalid_labels=True )
         logging.debug('... solution loaded')
         
         if get_duals: 
@@ -396,23 +415,17 @@ class OptimizationProblem(OptimizationObject):
             logging.info('resolving fixed-integer LP for duals')
             _fix_binary_variables(instance)
             
-            results,elapsed = self._solve_instance(instance, get_duals=get_duals)
-            self.solution_time+=elapsed
+            results, elapsed = self._solve_instance(instance, get_duals=get_duals)
+            self.solution_time += elapsed
             
             instance.load(results)
             logging.debug('... LP problem solved')    
         
         if self.stochastic_formulation:
-            self._scenario_tree.snapshotSolutionFromInstances(self._scenario_instances)
+            self._scenario_tree.snapshotSolutionFromInstances(self._scenario_instances)        
         
-        def get_objective(name):
-            try: return results.Solution.objective[name].value
-            except AttributeError: return results.Solution.objective['__default_objective__'].value
-        obj_name='objective' if (not self.stochastic_formulation) else 'MASTER'
-        
-        #print results.Solution.objective
-        #print get_duals,self.stochastic_formulation,obj_name
-        self.objective = get_objective(obj_name)
+        self.objective = get_objective(results,
+            name='MASTER' if self.stochastic_formulation else 'objective')
         
         #self.constraints = instance.active_components(pyomo.Constraint)
         #self.variables =   instance.active_components(pyomo.Var)
@@ -428,8 +441,18 @@ class OptimizationProblem(OptimizationObject):
         suffixes = ['dual'] if get_duals else []
 
         if not hasattr(self,'_opt_solver'):
-            self._opt_solver = cooprsolver.SolverFactory(solver)
-            # self._opt_solver.options.mipgap = '0.001'
+            kwds = {}            
+#            try: 
+#                if solver == 'gurobi':
+#                    import gurobipy
+#                    kwds['solver_io'] = 'python'
+#                elif solver == 'cplex':
+#                    import cplex
+#                    kwds['solver_io'] = 'python'         
+#            except ImportError: pass
+
+            self._opt_solver = cooprsolver.SolverFactory(solver, **kwds)
+            self._opt_solver.options.mipgap = user_config.mipgap
             
             if self._opt_solver is None: 
                 msg='solver "{}" not found by coopr'.format(solver)
@@ -443,9 +466,12 @@ class OptimizationProblem(OptimizationObject):
         if not keepFiles: logger.setLevel(current_log_level)
         self.solved = detect_status(results, self._opt_solver.name)
         
-        try: logging.debug('solution gap={}'.format(
-            results.Solution[0]['Gap']))
-        except: pass
+        if self.solved and not get_duals:
+            try: 
+                self.mipgap = results.Solution[0]['Gap']
+                logging.debug('solution gap={}'.format(self.mipgap))
+            except AttributeError: 
+                self.mipgap = None
         
         return results, elapsed
     
@@ -502,7 +528,7 @@ def value(variable):
     try: return variable.value
     except AttributeError: return variable #just a number
 
-def dual(constraint,index=None):
+def dual(constraint, index=None):
     '''Dual of optimization constraint, after the problem is solved.'''
     return constraint[index].dual
 
@@ -516,7 +542,7 @@ def detect_status(results, solver):
         success = False
     elif status_text == 'unknown':
         # this is an edge case encountered in resolves
-        success = len(results.Solution[0].Variable.keys()) > 0
+        success = len(results.Solution.Variable.keys()) > 0
     else:
         success = False
         
@@ -526,6 +552,14 @@ def detect_status(results, solver):
     return success
 
 
+def get_objective(results, name='objective'):
+    value = 0
+    try:
+        value = results.Solution.objective[name].value
+    except AttributeError:
+        objs = results.Solution.objective.values()
+        value = [obj['Value'] for obj in objs if 'Value' in obj][0]
+    return value
 
 
 class OptimizationError(Exception):
