@@ -13,7 +13,7 @@ import numpy as np
 import pandas as pd
 from optimization import (value, dual, OptimizationObject, 
     OptimizationProblem, OptimizationError)
-from commonscripts import (update_attributes, getattrL, flatten)
+from commonscripts import (update_attributes, getattrL, flatten, set_trace)
 from config import user_config
 
 class Load(OptimizationObject):
@@ -25,38 +25,44 @@ class Load(OptimizationObject):
     By setting `shedding_allowed`, the amount of power can become a variable,
         (bounded to be at most the scheduled amount).
     """
-    def __init__(self,kind='varying',name='',index=None,bus=None,schedule=None,
-                 shedding_allowed=False,
-                 cost_shedding=user_config.cost_load_shedding
-                 ):
+    def __init__(self,
+        name='', index=None, bus=None, schedule=None,
+        shedding_allowed=True,
+        cost_shedding=user_config.cost_load_shedding
+        ):
         update_attributes(self,locals()) #load in inputs
         self.init_optimization()
-    def power(self,time,evaluate=False):
-        if self.shedding_allowed:
-            power=self.get_variable('power',time,indexed=True)
-            if evaluate: power=value(power)
+        self.shedding_mode = False
+        
+    def power(self, time, evaluate=False):
+        if self.shedding_mode:
+            power = self.get_variable('power', time, indexed=True)
+            if evaluate: power = value(power)
             return power
         else:
-            return self.get_scheduled_ouput(time)
-    def shed(self,time,evaluate=False): return self.get_scheduled_ouput(time) - self.power(time,evaluate)
-    def cost(self,time): return self.cost_shedding*self.shed(time)
-    def cost_first_stage(self,times): return 0
-    def cost_second_stage(self,times): return sum(self.cost(time) for time in times)
+            return self.get_scheduled_output(time)
+    def shed(self, time, evaluate=False): 
+        return self.get_scheduled_output(time) - self.power(time,evaluate)
+    def cost(self,time): return self.cost_shedding * self.shed(time)
+    def cost_first_stage(self, times): return 0
+    def cost_second_stage(self, times):
+        return sum(self.cost(time) for time in times)
     def create_variables(self,times):
-        if self.shedding_allowed:
-            self.add_variable('power',index=times.set,low=0)
+        if self.shedding_mode:
+            self.add_variable('power', index=times.set, low=0)
     def create_constraints(self,times):
-        if self.shedding_allowed:
+        if self.shedding_mode:
             for time in times:
-                self.add_constraint('max_load_power',time,self.power(time)<=self.get_scheduled_ouput(time))
+                self.add_constraint('max_load_power',time,
+                    self.power(time) <= self.get_scheduled_output(time))
     def create_objective(self,times):
         return sum([ self.cost(time) for time in times])
 
     def __str__(self): return 'd{ind}'.format(ind=self.index)
     def __int__(self): return self.index
-    def iden(self,t):     return str(self)+str(t)
+    def iden(self, t): return str(self)+str(t)
 
-    def get_scheduled_ouput(self, time):
+    def get_scheduled_output(self, time):
         return float(self.schedule.ix[time])
 
 class Line(OptimizationObject):
@@ -64,9 +70,11 @@ class Line(OptimizationObject):
     A tranmission line. Currently the model
     only considers real power flow under normal conditions.
     """
-    def __init__(self,name='',index=None,From=None,To=None,X=0.05,Pmax=9999,Pmin=None,**kwargs):
+    def __init__(self, name='', index=None, frombus=None, tobus=None,
+        reactance=0.05, pmin=None, pmax=9999, **kwargs):
         update_attributes(self,locals()) #load in inputs
-        if self.Pmin is None: self.Pmin=-1*self.Pmax #reset default to be -Pmax
+        if self.pmin is None: 
+            self.pmin=-1*self.pmax   # default is -1*pmax
         self.init_optimization()
     def power(self,time): return self.get_variable('power',time,indexed=True)
     def price(self,time):
@@ -77,12 +85,13 @@ class Line(OptimizationObject):
     def create_constraints(self,times,buses):
         '''create the constraints for a line over all times'''
         busNames=getattrL(buses,'name')
-        iFrom,iTo=busNames.index(self.From),busNames.index(self.To)
+        iFrom,iTo=busNames.index(self.frombus),busNames.index(self.tobus)
         for t in times:
-            line_flow_ij=self.power(t) == (1/self.X) * (buses[iFrom].angle(t) - buses[iTo].angle(t))
+            line_flow_ij = self.power(t) == \
+                1/self.reactance * (buses[iFrom].angle(t) - buses[iTo].angle(t))
             self.add_constraint('line flow',t,line_flow_ij)
-            self.add_constraint('line limit high',t,self.power(t)<=self.Pmax)
-            self.add_constraint('line limit low',t,self.Pmin<=self.power(t))
+            self.add_constraint('line limit high',t,self.power(t)<=self.pmax)
+            self.add_constraint('line limit low',t,self.pmin<=self.power(t))
         return
     def __str__(self): return 'k{ind}'.format(ind=self.index)
     def __int__(self): return self.index
@@ -103,7 +112,8 @@ class Bus(OptimizationObject):
         self.init_optimization()
 
     def angle(self,time): return self.get_variable('angle',time,indexed=True)
-    def price(self,time): return dual(self.get_constraint('power balance',time))
+    def price(self,time): 
+        return dual(self.get_constraint('power balance', time))
     def Pgen(self,t,evaluate=False):
         if evaluate: return sum(value(gen.power(t)) for gen in self.generators)
         else: return sum(gen.power(t) for gen in self.generators)
@@ -170,44 +180,23 @@ class PowerSystem(OptimizationProblem):
     Other settings are inherited from `user_config`.
     '''
     def __init__(self, generators, loads, lines=None):
-        update_attributes(self,locals(),exclude=['generators','loads','lines']) #load in inputs
-        self.num_breakpoints           = user_config.breakpoints
-        self.load_shedding_allowed     = user_config.load_shedding_allowed
-        self.cost_load_shedding        = user_config.cost_load_shedding
-        self.dispatch_decommit_allowed = user_config.dispatch_decommit_allowed
-        self.reserve_fixed             = user_config.reserve_fixed
-        self.reserve_load_fraction     = user_config.reserve_load_fraction
-
+        # load in inputs
+        update_attributes(self, locals(),
+            exclude=['generators', 'loads', 'lines']) 
+        self.reserve_fixed = user_config.reserve_fixed
+        self.reserve_load_fraction = user_config.reserve_load_fraction
 
         if lines is None: lines=[]
 
-        buses=self.make_buses_list(loads,generators)
-        self.create_admittance_matrix(buses,lines)
+        buses = self.make_buses_list(loads, generators)
+        self.create_admittance_matrix(buses, lines)
         self.init_optimization()
 
-        self.add_children(buses,'buses')
-        self.add_children(lines,'lines')
-
-        #add system mode parameters to relevant components
-        self.set_load_shedding(self.load_shedding_allowed) #set load shedding
-        for load in loads:
-                try: load.cost_breakpoints = self.num_breakpoints
-                except AttributeError: pass #load has no cost model
-        for gen in generators:
-            gen.dispatch_decommit_allowed = self.dispatch_decommit_allowed
-            try: gen.cost_breakpoints = self.num_breakpoints
-            except AttributeError: pass #gen has no cost model
+        self.add_children(buses, 'buses')
+        self.add_children(lines, 'lines')
 
         self.is_stochastic = len(filter(lambda gen: gen.is_stochastic, generators))>0
-
-    def set_load_shedding(self,is_allowed):
-        '''set system mode for load shedding'''
-
-        self.load_shedding_allowed = is_allowed
-
-        for load in self.loads():
-            load.shedding_allowed=is_allowed
-            load.cost_shedding=self.cost_load_shedding
+        self.shedding_mode = False
 
     def make_buses_list(self, loads, generators):
         """
@@ -223,16 +212,24 @@ class PowerSystem(OptimizationProblem):
         busNameL=[]
         busNameL.extend(getattrL(generators,'bus'))
         busNameL.extend(getattrL(loads,'bus'))
-        busNameL=list(pd.unique(busNameL))
-        buses=[]
-        swingHasBeenSet=False
-        for b,busNm in enumerate(busNameL):
-            newBus=Bus(name=busNm,index=b)
+        busNameL = pd.Series(pd.unique(busNameL)).dropna().tolist()
+
+        if len(busNameL) == 0:
+            busNameL = [None]
+            
+        buses = []
+        swingHasBeenSet = False
+
+        for b, busNm in enumerate(busNameL):
+            newBus = Bus(name=busNm, index=b)
             for gen in generators:
-                if gen.bus==newBus.name: newBus.generators.append(gen)
-                if not swingHasBeenSet: newBus.isSwing=swingHasBeenSet=True
+                if gen.bus == newBus.name: 
+                    newBus.generators.append(gen)
+                if not swingHasBeenSet: 
+                    newBus.isSwing = swingHasBeenSet = True
             for ld in loads:
-                if ld.bus==newBus.name: newBus.loads.append(ld)
+                if ld.bus == newBus.name:
+                    newBus.loads.append(ld)
             buses.append(newBus)
         return buses
     def create_admittance_matrix(self, buses, lines):
@@ -248,10 +245,10 @@ class PowerSystem(OptimizationProblem):
         self.Bmatrix=np.zeros((nB,nB))
         namesL=[bus.name for bus in buses]
         for line in lines:
-            busFrom=buses[namesL.index(line.From)]
-            busTo=buses[namesL.index(line.To)]
-            self.Bmatrix[busFrom.index,busTo.index]+=-1/line.X
-            self.Bmatrix[busTo.index,busFrom.index]+=-1/line.X
+            busFrom = buses[namesL.index(line.frombus)]
+            busTo = buses[namesL.index(line.tobus)]
+            self.Bmatrix[busFrom.index,busTo.index] += -1/line.reactance
+            self.Bmatrix[busTo.index,busFrom.index] += -1/line.reactance
         for i in range(0,nB):
             self.Bmatrix[i,i]=-1*sum(self.Bmatrix[i,:])
     def loads(self): return flatten(bus.loads for bus in self.buses)
@@ -277,7 +274,7 @@ class PowerSystem(OptimizationProblem):
                 line.create_constraints(times, self.buses)
 
         # system reserve constraint
-        self._has_reserve = not self.load_shedding_allowed and \
+        self._has_reserve = not self.shedding_mode and \
             (self.reserve_fixed>0 or self.reserve_load_fraction>0)
         if self._has_reserve:
             for time in times:
@@ -296,13 +293,15 @@ class PowerSystem(OptimizationProblem):
     def total_scheduled_load(self):
         return sum([load.schedule for load in self.loads()]) 
     
-    
+    def total_scheduled_generation(self):
+        return sum(gen.schedule for gen in self.generators() if not gen.is_controllable)    
+        
     def get_generators_without_scenarios(self):
-        return filter(lambda gen: getattr(gen,'has_scenarios',False)==False, self.generators())
+        return filter(lambda gen: getattr(gen,'is_stochastic',False)==False, self.generators())
 
 
     def get_generator_with_scenarios(self):
-        gens = filter(lambda gen: getattr(gen,'has_scenarios',False), self.generators())
+        gens = filter(lambda gen: getattr(gen,'is_stochastic',False), self.generators())
         if len(gens)>1: raise NotImplementedError('Dont handle the case of multiple stochastic generators')
         elif len(gens)==0: return []
         else: return gens[0]
@@ -322,17 +321,16 @@ class PowerSystem(OptimizationProblem):
             stat = status[g]
             if sln.is_stochastic:
                 gen.finalstatus = dict(
-                    P =  sln.generators_power[g][tEnd],
-                    u =  sln.generators_status[g][tEnd],
-                    hoursinstatus = gen.gethrsinstatus(times.non_overlap(), stat))
-
+                    power=sln.generators_power[g][tEnd],
+                    status=sln.generators_status[g][tEnd],
+                    hoursinstatus=gen.gethrsinstatus(times.non_overlap(), stat)
+                    )
             else:
-                gen.finalstatus = gen.getstatus(tEndstr, times.non_overlap(), stat)
+                gen.finalstatus = gen.getstatus(tEndstr, 
+                    times.non_overlap(), stat)
         return
 
-    def set_initialconditions(self, initTime, stage_number, stage_solutions):
-        #first stage of problem already has initial time defined
-        if stage_number == 0: return
+    def set_initialconditions(self, initTime):
         for gen in self.generators():
             finalstatus = getattr(gen, 'finalstatus', {})
             if finalstatus:
@@ -341,6 +339,21 @@ class PowerSystem(OptimizationProblem):
         return
 
 
+    def solve_problem(self, times):
+        try: 
+            instance = self.solve()
+        except OptimizationError:
+            #re-do stage, with load shedding allowed
+            logging.critical('stage infeasible, re-run with load shedding.')
+            self.allow_shedding(times)
+            try:
+                instance = self.solve()
+            except OptimizationError:
+                scheduled, committed = self.debug_infeasibe(times)
+                set_trace()
+                raise OptimizationError('failed to solve with load shedding.')
+        return instance
+
     def resolve_stochastic_with_observed(self, instance, sln):
         s = sln.scenarios[0]        
         self._model = instance.active_components(pyomo.Block)[s]
@@ -376,34 +389,57 @@ class PowerSystem(OptimizationProblem):
         sln.observed_totalcost = sln.totalcost_generation
         return
 
-    def _allow_shed_resolve(self, sln):
-        self.set_load_shedding(True)
+    def _set_load_shedding(self, to_mode):
+        '''set system mode for load shedding'''
+        for load in self.loads():
+            load.shedding_mode = to_mode
+
+    def _set_gen_shedding(self, to_mode):
+        for gen in filter(lambda g: 
+            not g.is_controllable and g.sheddingallowed, self.generators()):
+            gen.shedding_mode = to_mode
+
+    
+    def allow_shedding(self, times, resolve=False):
+        self.shedding_mode = True
+        self._set_load_shedding(True)
+        self._set_gen_shedding(True)
+        
+        const_times = times.non_overlap() if resolve else times
         
         # make load power into a variable instead of a param
         for load in self.loads():
-            load.create_variables(sln.times)  # need all for the .set attrib
-            load.create_constraints(sln.times_non_overlap)
+            load.create_variables(times)  # need all for the .set attrib
+            load.create_constraints(const_times)
+        
+        for gen in filter(lambda g: 
+            getattr(g, 'shedding_mode', False), self.generators()):
+            gen.create_variables(times)
+            gen.create_constraints(const_times)
         
         # recalc the power balance constraint
         for bus in self.buses:
-            for time in sln.times:
+            for time in times:
                 bus._remove_component('power balance', time)
-            bus.create_constraints(sln.times_non_overlap, 
+            bus.create_constraints(const_times, 
                 self.Bmatrix, self.buses, include_children=False)
         
         # reset objective
         self._model.objective = None
-        self.create_objective(sln.times_non_overlap)
+        self.create_objective(const_times)
         # re-create system cost constraints 
-        self.create_constraints(sln.times_non_overlap, include_children=False)
+        self.create_constraints(const_times, include_children=False)        
+
+        # recreating all constraints would be simpler
+        # but would take a bit longer
+
+    def disallow_shedding(self):
+        # change shedding allowed flags for the next stage
+        self.shedding_mode = False
+        self._set_load_shedding(False)
+        self._set_gen_shedding(False)
+
 
-# recreating all constraints would be simpler, but would take a bit longer
-#        # reset objective
-#        self._model.objective = None
-#        self.create_objective(sln.times_non_overlap)
-#        # re-create constraints 
-#        self.create_constraints(sln.times_non_overlap)
-            
     def _resolve_problem(self, sln):
         times = sln.times_non_overlap
 
@@ -437,26 +473,56 @@ class PowerSystem(OptimizationProblem):
                 # allow faststart units to be started up to meet the load
                 self._unfix_variables()
                 self._fix_non_faststarts(sln.times)
+                logging.warning('allowing fast-starting units')
                 try: self.solve()
                 except OptimizationError:
-                    self._allow_shed_resolve(sln)
-                    self.solve()
+                    self._unfix_variables()
+                    self._fix_non_faststarts(sln.times, fix_power=False)
+                    logging.warning('''allowing non fast-starters
+                        to change power output''')
+                    try: self.solve()
+                    except OptimizationError:
+                        logging.warning('allowing load shedding')
+                        self.allow_shedding(sln.times, resolve=True)
+                        self.solve()
             else:
                 # just shed the un-meetable load and calculate cost later
-                self._allow_shed_resolve(sln)
+                self.allow_shedding(sln.times, resolve=True)
                 self.solve()
-            
         
         self.resolve_solution_time = self.solution_time
         self.solution_time = self.full_sln_time
         logging.info('resolved instance with observed values (in {}s)'.format(
             self.resolve_solution_time))
             
-    def _fix_non_faststarts(self, times):
-        # fix only non-faststart and ON statuses            
-        for gen in filter(lambda gen: not gen.faststart, self.generators()):
+    def _fix_non_faststarts(self, times, fix_power=True):
+        '''
+        fix non-faststart units - both power and status
+        (unless this is infeasible, then only fix status)
+        the idea is that fast-starts should be contributing power
+        only for system security, not economics
+        '''
+        for gen in filter(lambda gen: \
+            (not gen.faststart) and gen.is_controllable, self.generators()):
             for time in times:
-                status = gen.status(time)
-                if value(status) != 0 and not type(status)==bool:
-                    gen.status(time).fixed = True
-    
+                gen.status(time).fixed = True
+                if fix_power: gen.power(time).fixed = True
+    def debug_infeasibe(self, times):
+        scheduled = pd.DataFrame({
+            'load': self.total_scheduled_load().ix[times.strings.values], 
+            'generation': self.total_scheduled_generation().ix[times.strings.values]})
+        scheduled['net_required'] = scheduled['load'] - scheduled.generation
+        
+        print 'total scheduled\n', scheduled
+        gens = filter(lambda gen: \
+            gen.is_controllable and gen.initial_status == 1,
+            self.generators())
+        committed = pd.Series(dict(
+            Pmin=sum(gen.pmin for gen in gens),
+            Pmax=sum(gen.pmax for gen in gens),
+            rampratemin=sum(gen.rampratemin for gen in gens),
+            rampratemax=sum(gen.rampratemax for gen in gens),
+            ))
+        print 'total committed\n', committed
+        
+        return scheduled, committed
