@@ -120,6 +120,10 @@ class Solution(object):
     def _get_problem_info(self):
         self.solve_time = self.power_system.solution_time
         self.objective = float(value(self.power_system.objective))       
+        try: 
+            self.mipgap = self.power_system.mipgap
+        except AttributeError: 
+            self.mipgap = None
         
     def _get_outputs(self):
         self.generators_power = self.gen_time_df('power')
@@ -203,7 +207,7 @@ class Solution(object):
     def info_lines(self,t):
         lines=self.lines
         return ['line info:',
-             'connecting={}'.format(zip(getattrL(lines,'From'),getattrL(lines,'To'))),
+             'connecting={}'.format(zip(getattrL(lines,'frombus'),getattrL(lines,'tobus'))),
              'Pk={}'.format(self.get_values(lines,'power',t)),
              'price={}'.format(self.line_prices[str(t)])]
              
@@ -296,25 +300,24 @@ class Solution_ED(Solution):
         def niceTF(value): return 0 if value==0 else 1
         def nice_zeros(value): return 0 if value==0 else value
 
-        t=self.times[0]
-        generators=self.generators
-        output_loads= [load for load in self.loads if getattr(load,'kind',None) in ['bidding','shifting']]
-        fields,data=[],[]
-        components=flatten([generators,output_loads])
-        fields.append('name')
-        data.append(getattrL(components,'name'))
+        t = self.times.strings.index[0]
+        
+        out = pd.DataFrame(columns=['power', 'IC'], 
+            index=self.generators_power.columns) 
+        out['power'] = self.generators_power.ix[t]
+        out['IC'] = [nice_zeros(g.incrementalcost(self.times[0])) \
+            for g in self.generators]
 
         if user_config.dispatch_decommit_allowed:
-            fields.append('u')
-            data.append([niceTF( value(g.status(t)) and value(g.power(t))>0) for g in components])
+            # add status, but label all units at Pmin as OFF
+            Pmin = [gen.pmin for gen in self.generators]
+            out['status'] = self.generators_status.ix[t]            
+            out = out.drop('IC', axis=1)
 
-        fields.append('P')
-        data.append([nice_zeros(value(g.power(t))) for g in components])
-
-        fields.append('IC')
-        data.append([nice_zeros(g.incrementalcost(t)) for g in components])
-
-        writeCSV(fields,transpose(data),filename=full_filename('dispatch.csv'))
+        out.index = [gen.name for gen in self.generators]
+        out.index.name = 'name'
+        
+        out.to_csv(full_filename('dispatch.csv'))
 
 
 class Solution_OPF(Solution):
@@ -365,8 +368,8 @@ class Solution_OPF(Solution):
         writeCSV(fields,transpose(data),filename=full_filename('powerflow-generators.csv'))
 
         fields,data=[],[]
-        fields.append('from');  data.append(self.get_values(lines,'From'))
-        fields.append('to');  data.append(self.get_values(lines,'To'))
+        fields.append('from');  data.append(self.get_values(lines,'frombus'))
+        fields.append('to');  data.append(self.get_values(lines,'tobus'))
         fields.append('power'); data.append(self.get_values(lines,'power',t))
         fields.append('congestion shadow price'); data.append(self.line_prices[str(t)])
 
@@ -405,21 +408,21 @@ class Solution_UC_multistage(Solution_UC):
     Each element of the list :param:stage_solutions is a :class:`~results.Solution_UC` object.
 
     '''
-    def __init__(self,power_system, stage_times, stage_solutions):
-        update_attributes(self,locals(),exclude=['stage_solutions','stage_times'])
-        self._resolved = self.is_stochastic \
-            or user_config.deterministic_solve \
-            or user_config.perfect_solve
+    def __init__(self, power_system, stage_times, stage_solutions):
+        update_attributes(self, locals(),
+            exclude=['stage_solutions', 'stage_times'])
+        self._resolved = power_system.is_stochastic \
+            or user_config.deterministic_solve
 
         self.is_stochastic = any(sln.is_stochastic for sln in stage_solutions)
+        
         times = pd.concat([times.non_overlap().strings for times in stage_times]).index
-        self.times=TimeIndex(times)
+        self.times = TimeIndex(times)
         self.times.set_initial(stage_times[0].initialTime)
 
-        self.objective = self._sum_over('objective',stage_solutions)
-        self.solve_time = self._sum_over('solve_time',stage_solutions)
-        #self.active_constraints = sum([dual(c)!=0 for nm,c in constraints.items()])
-        #self.total_constraints = len(constraints)
+        self.objective = self._sum_over('objective', stage_solutions)
+        self.solve_time = self._sum_over('solve_time', stage_solutions)
+
         self._get_outputs(stage_solutions)
         self._get_costs(stage_solutions)
         self._get_prices(stage_solutions)
@@ -427,55 +430,56 @@ class Solution_UC_multistage(Solution_UC):
     def _sum_over(self,attrib,stage_solutions):
         return sum(getattr(sln, attrib) for sln in stage_solutions)
 
-    def _get_outputs(self,stage_solutions):
+    def _concat(self, attrib, slns):
+        return pd.concat([getattr(sln, attrib) for sln in slns])
+
+    def _get_outputs(self, slns):
         '''the outputs under observed wind'''
-        if self.is_stochastic or user_config.deterministic_solve:
-            self.generators_power = pd.concat([stage.observed_generator_power for stage in stage_solutions])
-            self.generators_status = pd.concat([stage.stage_generators_status for stage in stage_solutions])
-        else:
-            self.generators_power  = pd.concat([stage.generators_power for stage in stage_solutions])
-            self.generators_status = pd.concat([stage.generators_status for stage in stage_solutions])
+        self.generators_power = self._concat('generators_power', slns)
+        self.generators_status = self._concat('generators_status', slns)
+        if self._resolved:
+            self.expected_power = self._concat('expected_power', slns)
+            self.expected_status = self._concat('expected_status', slns)
+        elif user_config.perfect_solve:
+            self.expected_power = self.generators_power
+            self.expected_status = self.generators_status
 
-    def _get_costs(self,stage_solutions):
-        self.fuelcost_generation=self._sum_over('fuelcost_generation',stage_solutions)
-        self.totalcost_generation=self._sum_over('totalcost_generation',stage_solutions)
-        self.load_shed = self._sum_over('load_shed',stage_solutions)
-
-        if user_config.deterministic_solve:
-            self.expected_fuelcost_generation = self._sum_over('expected_fuelcost_generation',stage_solutions)
-            self.expected_totalcost_generation = self._sum_over('expected_totalcost_generation',stage_solutions)
-            self.expected_load_shed = self._sum_over('expected_load_shed',stage_solutions)
-
-        if not self.is_stochastic:
-            self.fuelcost_true=self._sum_over('fuelcost_true',stage_solutions)
-            self._get_cost_error()
+    def _get_costs(self, slns):
+        self.expected_cost = self.totalcost_generation = \
+            self._concat('expected_totalcost' \
+                if self._resolved else 'totalcost_generation', slns)
+        self.load_shed_timeseries = self._concat('load_shed_timeseries', slns)
+        self.load_shed = self.load_shed_timeseries.sum()
 
+        if self._resolved:
+            self.observed_cost = self.totalcost_generation = \
+                self._concat('observed_totalcost', slns)
+        elif user_config.perfect_solve:
+            self.observed_cost = self.totalcost_generation = self.expected_cost
 
 
     def info_cost(self):
-        resolved = (self.is_stochastic or user_config.deterministic_solve)
+        resolved = self._resolved
         expected = 'expected ' if resolved else ''
         observed = 'observed ' if resolved else ''
+        out = []
+        out.append('total {}generation cost = {}'.format(
+            expected, self.expected_cost.sum().sum()))
+        if resolved: 
+            out.append('total {}generation cost = {}'.format(
+                observed, self.observed_cost.sum().sum()))
 
-        out = [
-        '{}objective cost={}'.format(expected, self.objective),
-        'total {}generation costs={}'.format(observed, self.totalcost_generation),
-        'linearized {}fuel cost of generation={}'.format(observed, self.fuelcost_generation),
-        ]
-        if user_config.deterministic_solve:
-            out.extend([
-                'total expected generation cost = {}'.format(self.expected_totalcost_generation)
-            ])
-        if not resolved:
-            out.extend([
-                ' non-linearized cost of generation={}'.format(self.fuelcost_true),
-                'percentage difference\t\t={diff:.2%}'.format(diff=self.costerror),
-            ])
         return out
 
     def _get_prices(self,stage_solutions):
         self.lmps={}
-        for stage in stage_solutions: self.lmps.update(stage.lmps)
+        try: 
+            for stage in stage_solutions: self.lmps.update(stage.lmps)
+        except: 
+            # no lmps in stochastic solutions right now
+            pass
+    
+            
     def show(self):
         out=[]
         out.extend(self.info_status())
@@ -485,9 +489,35 @@ class Solution_UC_multistage(Solution_UC):
     def info_generators(self): return []
     def info_loads(self): return []
     def info_status(self):
-        return ['solved multistage problem in a total solver time of {time:0.4f} sec'.format(time=self.solve_time)]
+        return ['solved multistage problem in a total solver time ' + \
+            'of {time:0.4f} sec'.format(time=self.solve_time)]
     def info_shedding(self):
-        return ['total load shed={}MW'.format(self.load_shed) if self.load_shed>0 else '']
+        return ['total load shed={}MW'.format(self.load_shed) \
+            if self.load_shed > 0.01 else '']
+
+
+class MultistageStandalone(Solution_UC_multistage):
+    def __init__(self, power_system, stage_times, store):
+        self.power_system = power_system
+        self.is_stochastic = power_system.is_stochastic
+        self._resolved = self.is_stochastic or user_config.deterministic_solve
+        times = pd.concat([times.non_overlap().strings for times in stage_times]).index
+        self.times=TimeIndex(times)
+        self.times.set_initial(stage_times[0].initialTime)
+
+        self.expected_cost = self.totalcost_generation = store['expected_cost']
+
+        if self._resolved or user_config.perfect_solve:
+            self.observed_cost = self.totalcost_generation = \
+                store['observed_cost']
+            
+            
+        self.generators_power = store['power']
+        self.generators_status = store['status']
+        self.load_shed_timeseries = store['load_shed']
+
+        self.load_shed = store['load_shed'].sum()
+        self.solve_time = store['solve_time'].sum()
 
 
 class Solution_Stochastic(Solution):
@@ -496,14 +526,14 @@ class Solution_Stochastic(Solution):
         self._setup_powersystem()
         
         self.scenarios=sorted(self.power_system._scenario_instances.keys())
-        self.stage_date = times.strings.index[0].date()
+        self.stage_start = times.strings.index[0]
         
         gen = self.power_system.get_generator_with_scenarios()
         
         try: # single stage setup
             self.probability = gen.scenario_values.probability
         except AttributeError:  # staged setup 
-            self.probability = gen.scenario_values[self.stage_date].probability
+            self.probability = gen.scenario_values[self.stage_start].probability
             
         self.probability.index = self.scenarios
         
@@ -540,8 +570,7 @@ class Solution_Stochastic(Solution):
         if resolve:
             # observed generator power
             # resolved on the first scenario instance
-            # -- no more scenario labeling is needed            
-            self.expected_status = self.generators_status.copy()
+            # -- no more scenario labeling is needed
             self.generators_power = self.gen_time_df('power', None)
             self.generators_status = self.gen_time_df('status', None)
             
@@ -549,8 +578,9 @@ class Solution_Stochastic(Solution):
             self.generators_power_scenarios = self.stg_panel('power')
             self.generators_status_scenarios = \
                 _correct_status(self.stg_panel('status'))
-            self.generators_status = \
+            self.expected_status = \
                 self.generators_status_scenarios[self.scenarios[0]]
+            self.generators_status = self.expected_status.copy()
             self.expected_power = self._calc_expected(
                 self.generators_power_scenarios)
         return
@@ -619,44 +649,12 @@ class Solution_Stochastic_UC(Solution_Stochastic):
         self.generators_status.to_csv(full_filename('commitment-status.csv'))
 
 
-class MultistageStandalone(Solution_UC_multistage):
-    def __init__(self, power_system, stage_times, store):
-        self.power_system = power_system
-        self.is_stochastic = power_system.is_stochastic
-        self._resolved = self.is_stochastic \
-            or user_config.deterministic_solve \
-            or user_config.perfect_solve
-        times = pd.concat([times.non_overlap().strings for times in stage_times]).index
-        self.times=TimeIndex(times)
-        self.times.set_initial(stage_times[0].initialTime)
-
-        self.expected_cost = self.totalcost_generation = store['expected_cost']
-        if self._resolved:
-            self.observed_cost = self.totalcost_generation = \
-                store['observed_cost']
-        self.generators_power = store['power']
-        self.generators_status = store['status']
-        self.load_shed_timeseries = store['load_shed']
-
-        self.load_shed = store['load_shed'].sum()
-        self.solve_time = store['solve_time'].sum()
-
-    def info_cost(self):
-        resolved = self._resolved
-        expected = 'expected ' if resolved else ''
-        observed = 'observed ' if resolved else ''
-
-        out = ['total {}generation cost={}'.format(expected, self.expected_cost.sum().sum())]
-        if resolved: out.append(
-            'total {}generation cost={}'.format(observed, self.observed_cost.sum().sum()))
-
-        return out
-
-
-
 def _correct_status(status):
     # correct for strange solver values returned on resolve
     status[status > 0.99] = 1
+    
+    test_status = status.to_frame() if hasattr(status, 'to_frame') else status    
+    assert(not ((test_status<=0.99) & (test_status>=0.01)).any().any() )
     return status.astype(int)
 
 
