@@ -8,27 +8,26 @@ problems and solving them.
 import sys, os, logging, subprocess
 import time as timer
 import argparse
+import pdb
 from optimization import OptimizationError
 import get_data, powersystems, stochastic, results
 from config import user_config
-from standalone import (store_state, load_state, store_times,
-    get_storage, wipe_storage, repack_storage)
-from commonscripts import joindir, StreamToLogger
+from commonscripts import joindir, StreamToLogger, set_trace
+from standalone import store_times, init_store, get_storage, repack_storage
 
-def _get_store_filename():
+def _set_store_filename(pid=None):
     fnm = 'stage-store.hd5'
     if user_config.output_prefix:
-        fnm = '{}-{}'.format(user_config._pid, fnm)
+        fnm = '{}-{}'.format(pid, fnm)
 
     user_config.store_filename = joindir(user_config.directory, fnm)
 
-def solve_multistage(power_system, times, scenario_tree):
-    _get_store_filename()
+def solve_multistage_standalone(power_system, times, scenario_tree, data):
 
-    wipe_storage()
-    stage_times=times.subdivide(user_config.hours_commitment, user_config.hours_overlap)
+    stage_times = times.subdivide(
+        user_config.hours_commitment, user_config.hours_overlap)
 
-    storage = store_state(power_system, stage_times, None)
+    storage = init_store(power_system, stage_times, data)
 
     for stg,t_stage in enumerate(stage_times):
         logging.info('Stage starting at {}'.format(t_stage.Start.date()))
@@ -50,6 +49,7 @@ def solve_multistage(power_system, times, scenario_tree):
 
 def standaloneUC():
     '''the hook for the ``standalone_minpower`` script'''
+    from standalone import store_state, load_state
 
     parser = argparse.ArgumentParser()
     parser.add_argument('directory', type=str, help='the problem direcory')
@@ -62,45 +62,16 @@ def standaloneUC():
     user_config.directory = args.directory
     if args.pid:
         user_config.output_prefix = True
-        user_config._pid = args.pid
+    
+    _set_store_filename(args.pid)
 
-    _get_store_filename()
-
-    _setup_logging()
+    _setup_logging(args.pid)
 
     # load stage data
     power_system, times, scenario_tree = load_state()
 
-    try:
-        sln, instance = create_solve_problem(power_system, times, scenario_tree, multistage=True, stage_number=stg)
-    except OptimizationError:
-        #re-do stage, with load shedding allowed
-        logging.critical('stage infeasible, re-running with load shedding.')
-        power_system.reset_model()
-        power_system.set_load_shedding(True)
-        try:
-            sln, instance = create_solve_problem(power_system, times, scenario_tree, multistage=True, stage_number=stg, rerun=True)
-        except OptimizationError:
-            raise OptimizationError('failed to solve, even with load shedding.')
-        power_system.set_load_shedding(False)
+    sln = create_solve_problem(power_system, times, scenario_tree, stage_number=stg)
 
-    logging.debug('solved... get results')
-
-    # resolve with observed power and fixed status 
-    if sln.is_stochastic:        
-        power_system.resolve_stochastic_with_observed(instance, sln)
-    elif user_config.deterministic_solve:
-        power_system.resolve_determinisitc_with_observed(sln)
-
-    if sln.load_shed_timeseries.sum() > 0.01:
-        logging.debug('shed {}MW in resolve of stage'.format(
-            sln.load_shed_timeseries.sum()))
-    
-
-    power_system.get_finalconditions(sln)
-
-    power_system.set_load_shedding(False)
-    sln.stage_number = stg
     store = store_state(power_system, times, sln)
     store.flush()
 
@@ -117,21 +88,30 @@ def solve_problem(datadir='.',
     The problem type is determined by the data.
     All options are set within `user_config`.
     """
-
-    _setup_logging()
     user_config.directory = datadir
+    
+    pid = os.getpid()
+    _set_store_filename(pid)
+    _setup_logging(pid)
+    
     start_time = timer.time()
     logging.debug('Minpower reading {}'.format(datadir))
-    generators,loads,lines,times,scenario_tree=get_data.parsedir()
+    generators, loads, lines, times, scenario_tree, data = get_data.parsedir()
     logging.debug('data read')
-    power_system=powersystems.PowerSystem(generators,loads,lines)
+    power_system = powersystems.PowerSystem(generators,loads,lines)
 
     logging.debug('power system initialized')
     if times.spanhrs <= user_config.hours_commitment + user_config.hours_overlap:
-        solution, instance = create_solve_problem(power_system, times, scenario_tree)
+        solution = create_solve_problem(power_system, times, scenario_tree)
     else: #split into multiple stages and solve
-        stage_solutions, stage_times = solve_multistage(power_system, times, scenario_tree)
-        solution = results.make_multistage_solution(power_system, stage_times, stage_solutions)
+        if user_config.standalone:
+            stage_solutions, stage_times = solve_multistage_standalone(
+                power_system, times, scenario_tree, data)
+        else:
+            stage_solutions, stage_times = solve_multistage(
+                power_system, times, scenario_tree, data)
+        solution = results.make_multistage_solution(
+            power_system, stage_times, stage_solutions)
 
     if shell:
         if user_config.output_prefix:
@@ -146,25 +126,64 @@ def solve_problem(datadir='.',
     logging.info('total time: {}s'.format(timer.time()-start_time))
     return solution
 
+
+def solve_multistage(power_system, times, scenario_tree=None, data=None):
+    stage_times = times.subdivide(
+        user_config.hours_commitment, user_config.hours_overlap)
+
+    stage_solutions = []
+
+    for stg, t_stage in enumerate(stage_times):
+        logging.info('Stage starting at {}'.format(t_stage.Start.date()))
+        # solve
+        solution = create_solve_problem(
+            power_system, t_stage, scenario_tree, stg)
+        # add to stage solutions
+        stage_solutions.append(solution)
+        # reset model
+        power_system.reset_model()
+        # set inital state for next stage
+        if stg < len(stage_times)-1:
+            power_system.set_initialconditions(stage_times[stg+1].initialTime)
+
+    return stage_solutions, stage_times
+
+
 def create_solve_problem(power_system, times, scenario_tree=None,
-    multistage=False, stage_number=None, rerun=False):
+    stage_number=None, rerun=False):
     '''create and solve an optimization problem.'''
-
-    if user_config.problemfile:
-        user_config.problemfile = joindir(
-            user_config.directory, 'problem-formulation.lp')
-
+    
     create_problem(power_system, times, scenario_tree,
-        multistage, stage_number, rerun)
+        stage_number, rerun)
+    
+    instance = power_system.solve_problem(times)
 
-    instance = power_system.solve()
+    logging.debug('solved... get results')
 
-    solution=results.make_solution(power_system,times,datadir=user_config.directory)
+    sln = results.make_solution(power_system, times)
+    
+    power_system.disallow_shedding()
 
-    return solution, instance
+    # resolve with observed power and fixed status
+    if sln.is_stochastic:
+        power_system.resolve_stochastic_with_observed(instance, sln)
+    elif user_config.deterministic_solve:
+        power_system.resolve_determinisitc_with_observed(sln)
+   
+    if sln.load_shed_timeseries.sum() > 0.01:
+        logging.debug('shed {}MW in resolve of stage'.format(
+            sln.load_shed_timeseries.sum()))
+
+    if len(times)>1:
+        power_system.get_finalconditions(sln)
+
+        power_system.disallow_shedding()
+        sln.stage_number = stage_number
+    
+    return sln
 
 def create_problem(power_system, times, scenario_tree=None,
-    multistage=False, stage_number=None, rerun=False):
+    stage_number=None, rerun=False):
     """Create an optimization problem."""
 
     logging.debug('initialized problem')
@@ -175,35 +194,33 @@ def create_problem(power_system, times, scenario_tree=None,
     power_system.create_constraints(times)
     logging.debug('created constraints')
 
-    if scenario_tree is not None and not rerun:
-        if multistage: # multiple time stages
-            gen = power_system.get_generator_with_scenarios()
-            tree = stochastic.construct_simple_scenario_tree(
-                gen.scenario_values[times.startdate]['probability'].values.tolist(),
-                time_stage=stage_number)
-            logging.debug('constructed tree for stage %i'%stage_number)
-        else: tree = scenario_tree
-        stochastic.define_stage_variables(tree,power_system,times)
-        # ipython_shell()
-        # %prun from minpower import stochastic; stochastic.create_problem_with_scenarios(power_system,times, tree, 24, 12, stage_number=0)
+    if scenario_tree is not None and sum(scenario_tree.shape)>0 and not rerun:
+        gen = power_system.get_generator_with_scenarios()
+        tree = stochastic.construct_simple_scenario_tree(
+            gen.scenario_values[times.Start]['probability'].values.tolist(),
+            time_stage=stage_number)
+
+        logging.debug('constructed tree for stage %i'%stage_number)
+
+        stochastic.define_stage_variables(tree, power_system, times)
         power_system = stochastic.create_problem_with_scenarios(
-            power_system, times, tree, 
-            user_config.hours_commitment, 
-            user_config.hours_overlap, 
+            power_system, times, tree,
+            user_config.hours_commitment,
+            user_config.hours_overlap,
             stage_number=stage_number)
     return
 
-def _setup_logging():
+def _setup_logging(pid=None):
     ''' set up the logging to report on the status'''
     kwds = dict(
-        level=user_config.logging_level, 
+        level=user_config.logging_level,
         datefmt='%Y-%m-%d %H:%M:%S',
         format='%(asctime)s %(levelname)s: %(message)s')
     if user_config.logging_filename:
         kwds['filename'] = user_config.logging_filename
     if user_config.output_prefix:
-        kwds['filename'] = joindir(user_config.directory, 
-            '{}.pylog'.format(user_config._pid))
+        kwds['filename'] = joindir(user_config.directory,
+            '{}.pylog'.format(pid))
     if (user_config.logging_level > 10) and (not 'filename' in kwds):
         # don't log the time if debugging isn't turned on
         kwds['format'] = '%(levelname)s: %(message)s'
@@ -236,56 +253,88 @@ def main():
         default=user_config.hours_overlap,
         help='number hours to overlap commitments in a rolling UC')
 
-    parser.add_argument('--reserve_fixed', type=float,
+    parser.add_argument('--mipgap',  type=float,
+        default=user_config.mipgap,
+        help='the MIP gap solution tolerence')
+
+    reserve = parser.add_argument_group('Reserve',
+        'Does the system require reserve? The default is no reserve.')
+    reserve.add_argument('--reserve_fixed', type=float,
         default=user_config.reserve_fixed,
-        help='The static amount of reserve required at all times (in MW)')
-    parser.add_argument('--reserve_load_fraction', type=float,
+        help='the static amount of reserve required at all times (in MW)')
+    reserve.add_argument('--reserve_load_fraction', type=float,
         default=user_config.reserve_load_fraction,
         help='fraction of the total system load which is required as reserve')
 
-    parser.add_argument('--problemfile',action="store_true",
-        default=user_config.problem_filename,
-        help='flag to write the problem formulation to a problem-formulation.lp file -- useful for debugging')
     parser.add_argument('--duals','-d',action="store_true",
         default=user_config.duals,
         help='flag to get the duals, or prices, of the optimization problem')
     parser.add_argument('--dispatch_decommit_allowed', action="store_true",
         default=user_config.dispatch_decommit_allowed,
         help='flag to allow de-commitment of units in an ED -- useful for getting initial conditions for UCs')
-    parser.add_argument('--logfile','-l',type=str,
-        default=user_config.logging_filename,
-        help='log file, default is to log to terminal')
-    parser.add_argument('--scenarios',type=int,
-        default=user_config.scenarios,
-        help='limit the number of scenarios to N')
 
-    parser.add_argument('--deterministic_solve', action='store_true',
+    parser.add_argument('--wind_multiplier', type=float,
+        default=user_config.wind_multiplier,
+        help='scale the wind power by this factor')
+    
+    stochastic = parser.add_argument_group('Stochastic UC',
+        'options to modify the behavior of a stochastic problem')
+    stochastic.add_argument('--scenarios',type=int,
+        default=user_config.scenarios,
+        help='limit the number of scenarios to N')        
+    stochastic.add_argument('--faststart_resolve', action='store_true',
+        default=user_config.faststart_resolve,
+        help="""allow faststart units which are off to be
+                started up during resolve with observed wind values""")
+
+    stochastic_mode = stochastic.add_mutually_exclusive_group()
+    stochastic_mode.add_argument('--deterministic_solve', action='store_true',
         default=user_config.deterministic_solve,
         help='solve a stochastic problem deterministically using the forecast_filename paramter')
-    parser.add_argument('--perfect_solve', action='store_true',
+    stochastic_mode.add_argument('--perfect_solve', action='store_true',
         default=user_config.perfect_solve,  #False
         help='solve a stochastic problem with perfect information')
-    parser.add_argument('--scenarios_directory', type=str,
+    stochastic_mode.add_argument('--scenarios_directory', type=str,
         default=user_config.scenarios_directory,
         help='override scenarios directory for stochastic problem')
-    parser.add_argument('--faststart_resolve', action='store_true',
-        default=False,
-        help="""allow faststart units which are off to be 
-                started up during resolve""")
+        
 
+    parser.add_argument('--standalone', '-m', action="store_true", 
+        default=user_config.standalone,
+        help='Make each multi-day commitment its own subprocess (helps with memory issues).')
     parser.add_argument('--output_prefix','-p', action="store_true",
         default=user_config.output_prefix,
-        help = 'Prefix all results files with the process id (for a record of simulataneous solves)')
+        help='Prefix all results files with the process id (for a record of simulataneous solves)')
 
-    parser.add_argument('--profile',action="store_true",
+    debugging = parser.add_argument_group('Debugging tools')
+    debugging.add_argument('--debugger',action="store_true",
+        default=user_config.debugger,
+        help='use pdb when an error is raised')
+    debugging.add_argument('--logfile','-l',type=str,
+        default=user_config.logging_filename,
+        help='log file, default is to log to terminal')
+    debugging.add_argument('--problemfile',action="store_true",
+        default=user_config.problem_filename,
+        help='flag to write the problem formulation to a problem-formulation.lp file -- useful for debugging')    
+    debugging.add_argument('--profile',action="store_true",
         default=False,
         help='run cProfile and output to minpower.profile')
-    parser.add_argument('--error','-e',action="store_true",
-        default=False,
-        help='redirect error messages to the standard output (useful for debugging on remote machines)')
-    parser.add_argument('--debugger',action="store_true",
-        default=False,
-        help='use pudb when an error is raised')
+    
+    constraints = parser.add_argument_group('Ignore/relax constraints',
+        'Ignore or relax sets of constraints to allow for feasible solutions.')
+    constraints.add_argument('--load_shedding_allowed', action='store_true',
+        default=user_config.load_shedding_allowed,
+        help='''Allow load shedding at a high cost (shedding is always a 
+            fallback if the system cannot meet the load).''')
+    constraints.add_argument('--ignore_minhours_constraints', action="store_true",
+        default=user_config.ignore_minhours_constraints,
+        help='drop the min up/down time constraints on the generators')
+    constraints.add_argument('--ignore_ramping_constraints', action="store_true",
+        default=user_config.ignore_ramping_constraints,
+        help='drop the min ramping power constraints on the generators')
+    
+    # NOTE - don't let defaults creep into this defenition
+    # that makes resetting the defaults during testing very hard 
 
     #figure out the command line arguments
     args = parser.parse_args()
@@ -297,8 +346,7 @@ def main():
         raise OSError(msg)
 
     user_config.update(vars(args))
-    if user_config.output_prefix: user_config._pid = os.getpid()
-
+    
     if args.profile:
         print 'run profile'
         import cProfile
@@ -310,9 +358,10 @@ def main():
         #solve the problem with those arguments
         try: solve_problem(directory)
         except:
-            if args.error:
-                print 'There was an error:'
-                traceback.print_exc(file=sys.stdout)
+            if args.debugger:
+                __, __, tb = sys.exc_info()
+                traceback.print_exc()
+                pdb.post_mortem(tb)
             else: raise
 
 # for use in dev
