@@ -1,11 +1,12 @@
-from pandas import Series
-from minpower import optimization,powersystems,schedule,solve,config
+from pandas import Series, DataFrame
+from minpower import powersystems,schedule,solve
 from minpower.generators import Generator
 from minpower.config import user_config
-from minpower.optimization import value
+from minpower.optimization import value, OptimizationError
 
 import nose
-from nose.tools import istest, with_setup, eq_, raises, set_trace
+from nose.tools import istest, with_setup, raises, set_trace
+from nose.tools import eq_ as assert_equal
 
 
 import logging
@@ -15,14 +16,22 @@ logging.basicConfig(
 
 
 
-user_config_defaults = user_config.copy()
+default_config = user_config.copy()
 
 def get_duals():
     user_config.duals = True
-#def reset_config():
-    # sadly this does not work
-#   user_config = user_config_defaults.copy()
-
+    
+def reset_config():
+    # note - can't set a new user_config, because it would be a local variable
+    #   instead, we update it with the defaults
+    #   this only works if we aren't adding anything extra to the user_config
+    user_config.update(default_config)
+    try: assert(user_config == default_config)
+    except:
+        print DataFrame(dict(
+            config=Series(user_config), 
+            default=Series(default_config) ))
+        raise
 
 singletime=schedule.just_one_time()
 
@@ -40,11 +49,11 @@ def assertAlmostEqual(x,y):
     #return [singlebus]    
 
 def make_cheap_gen(**kwargs):
-    return Generator(name='cheap gen', costcurvestring='{}P'.format(gen_costs['cheap']), **kwargs)
+    return Generator(name='cheap gen', costcurveequation='{}P'.format(gen_costs['cheap']), **kwargs)
 def make_mid_gen(**kwargs):
-    return Generator(name='middle-range gen', costcurvestring='{}P'.format(gen_costs['mid']), **kwargs)    
+    return Generator(name='middle-range gen', costcurveequation='{}P'.format(gen_costs['mid']), **kwargs)    
 def make_expensive_gen(**kwargs):
-    if 'costcurvestring' not in kwargs: kwargs['costcurvestring']='{}P'.format(gen_costs['expensive'])
+    if 'costcurveequation' not in kwargs: kwargs['costcurveequation']='{}P'.format(gen_costs['expensive'])
     return Generator(name='expensive gen', **kwargs)    
 def make_loads_times(Pd=200,Pdt=None,**kwargs):
     if Pdt is None:
@@ -58,9 +67,8 @@ def make_loads_times(Pd=200,Pdt=None,**kwargs):
     
     return dict(loads=loads,times=times)
 
-def solve_problem(generators,loads=None,times=None, gen_init=None, lines=None):
-    
-    
+def solve_problem(generators, loads=None, times=None, 
+    gen_init=None, lines=None, do_reset_config=True):
     
     if lines is None: lines=[]
 
@@ -72,14 +80,28 @@ def solve_problem(generators,loads=None,times=None, gen_init=None, lines=None):
     
     power_system=powersystems.PowerSystem(generators,loads,lines)
     solve.create_problem(power_system,times)
-    power_system.solve()
-    
-    config.user_config = user_config_defaults
+
+    try: 
+        instance = power_system.solve()
+    except OptimizationError:
+        #re-do stage, with load shedding allowed
+        logging.critical('stage infeasible, re-running with load shedding.')
+        power_system.allow_shedding(times)
+        try:
+            instance = power_system.solve()
+        except OptimizationError:
+            scheduled, committed = power_system.debug_infeasibe(times)
+            set_trace()
+            raise OptimizationError('failed to solve, even with load shedding.')
+        
+
+    if do_reset_config:
+        reset_config()
     
     if power_system.solved:
         power_system.update_variables()
     else:
         #logging.critical( [g.power[times.initialTime] for g in generators] )
         power_system.write_model('problem.lp')
-        raise optimization.OptimizationError('infeasible problem, wrote to problem.lp')
+        raise OptimizationError('infeasible problem, wrote to problem.lp')
     return power_system,times
